#define TK_ABORT                           1
#define TK_AGG_FUNCTION                    2
#define TK_ALL                             3
#define TK_AND                             4
#define TK_AS                              5
#define TK_ASC                             6
#define TK_BEGIN                           7
#define TK_BETWEEN                         8
#define TK_BITAND                          9
#define TK_BITNOT                         10
#define TK_BITOR                          11
#define TK_BY                             12
#define TK_CHECK                          13
#define TK_CLUSTER                        14
#define TK_COLUMN                         15
#define TK_COMMA                          16
#define TK_COMMENT                        17
#define TK_COMMIT                         18
#define TK_CONCAT                         19
#define TK_CONFLICT                       20
#define TK_CONSTRAINT                     21
#define TK_COPY                           22
#define TK_CREATE                         23
#define TK_DEFAULT                        24
#define TK_DELETE                         25
#define TK_DELIMITERS                     26
#define TK_DESC                           27
#define TK_DISTINCT                       28
#define TK_DOT                            29
#define TK_DROP                           30
#define TK_END                            31
#define TK_END_OF_FILE                    32
#define TK_EQ                             33
#define TK_EXCEPT                         34
#define TK_EXPLAIN                        35
#define TK_FAIL                           36
#define TK_FLOAT                          37
#define TK_FROM                           38
#define TK_FUNCTION                       39
#define TK_GE                             40
#define TK_GLOB                           41
#define TK_GROUP                          42
#define TK_GT                             43
#define TK_HAVING                         44
#define TK_ID                             45
#define TK_IGNORE                         46
#define TK_ILLEGAL                        47
#define TK_IN                             48
#define TK_INDEX                          49
#define TK_INSERT                         50
#define TK_INTEGER                        51
#define TK_INTERSECT                      52
#define TK_INTO                           53
#define TK_IS                             54
#define TK_ISNULL                         55
#define TK_KEY                            56
#define TK_LE                             57
#define TK_LIKE                           58
#define TK_LIMIT                          59
#define TK_LP                             60
#define TK_LSHIFT                         61
#define TK_LT                             62
#define TK_MINUS                          63
#define TK_NE                             64
#define TK_NOT                            65
#define TK_NOTNULL                        66
#define TK_NULL                           67
#define TK_OFFSET                         68
#define TK_ON                             69
#define TK_OR                             70
#define TK_ORDER                          71
#define TK_PLUS                           72
#define TK_PRAGMA                         73
#define TK_PRIMARY                        74
#define TK_REM                            75
#define TK_REPLACE                        76
#define TK_ROLLBACK                       77
#define TK_RP                             78
#define TK_RSHIFT                         79
#define TK_SELECT                         80
#define TK_SEMI                           81
#define TK_SET                            82
#define TK_SLASH                          83
#define TK_SPACE                          84
#define TK_STAR                           85
#define TK_STRING                         86
#define TK_TABLE                          87
#define TK_TEMP                           88
#define TK_TRANSACTION                    89
#define TK_UMINUS                         90
#define TK_UNCLOSED_STRING                91
#define TK_UNION                          92
#define TK_UNIQUE                         93
#define TK_UPDATE                         94
#define TK_USING                          95
#define TK_VACUUM                         96
#define TK_VALUES                         97
#define TK_WHERE                          98
